      SUBROUTINE NERS_FETCH ( NERS, IUER )
! ************************************************************************
! *                                                                      *
! *   Routine  NERS_FETCH
! *                                                                      *
! *  ### 16-JUN-2016   NERS_FETCH   v2.1 (c) L. Petrov  26-JUN-2018 ###  *
! *                                                                      *
! ************************************************************************
      IMPLICIT   NONE 
      INCLUDE   'ners.i'
      INCLUDE   'astro_constants.i'
      TYPE     ( NERS__TYPE ) :: NERS
      INTEGER*4  IUER 
      CHARACTER  STR*128, COM_STR*512, TEMP_NERS_FCS*128, TEMP_NERS_LOG*128, &
     &           TEMP_NERS_LOCK*128, CONN_TIMEOUT_STR*12, &
     &           READ_TIMEOUT_STR*12, LOCK_FILE*128
      INTEGER*4  IS, J1, J2, J3, PID_LOCK, M_WAIT_TRIES, IER
      INTEGER*4, EXTERNAL :: ILEN, I_LEN, GET_PID, KILL
      LOGICAL*4  LEX
      REAL*8,    EXTERNAL :: TIME
      INTEGER*4, EXTERNAL :: SYSTEM, GETPID
!
      WRITE ( UNIT=STR(1:5), FMT='(I5.5)' ) GETPID()
      TEMP_NERS_FCS  = TRIM(NERS%CNF%FCS_FILE)//'_'//STR(1:5)
      TEMP_NERS_LOG  = '/tmp/ners__'//STR(1:5)//'.log'
      TEMP_NERS_LOCK = '/tmp/ners__'//STR(1:5)//'.lock'
!
! --- Wait M_WAIT_TRIES cycles of NERS__WAIT_TIM seconds each
!
      M_WAIT_TRIES = (NERS%CNF%CONN_TIMEOUT + NERS%CNF%READ_TIMEOUT)/NERS__WAIT_TIM + 1
      DO 410 J1=1,M_WAIT_TRIES
!
! ------ Check whether the lock file exists
!
         INQUIRE ( FILE=TEMP_NERS_LOCK, EXIST=LEX )
         IF ( LEX ) THEN
!
! ----------- The lock file exists. Let us check whether the PID that created it
! ----------- is still running
!
              CALL RD_TEXT ( TEMP_NERS_LOCK, 1, STR, IS, IER )
              IF ( IER == 0 ) THEN
!
! ---------------- Try to gently "kill" that process
!
                   CALL CHIN ( STR, PID_LOCK )
                   IS = KILL ( %VAL(PID_LOCK), %VAL(0) )
!
! ---------------- Nope! the process does not exist
!
                   IF ( IS .NE. 0 ) GOTO 810
              END IF
              CALL LIB$WAIT ( NERS__WAIT_TIM )
            ELSE
!
! ----------- Does not exist. End of checks
!
              GOTO 810
         END IF
 410  CONTINUE 
      IF ( IER .NE. 0 ) THEN
           CALL CLRCH ( STR )
           WRITE ( UNIT=STR, FMT=* ) M_WAIT_TRIES*NERS__WAIT_TIM
           CALL ERR_LOG ( 4341, IUER, 'NERS_FETCH', 'Waited for '// &
     &          TRIM(STR)//' seconds to release lock file '// &
     &          TRIM(TEMP_NERS_LOCK)//' but the file is still locked' )
           RETURN 
      END IF      
 810  CONTINUE 
!
! --- Write the lock file
!
      WRITE ( UNIT=STR(1:5), FMT='(I5.5)' ) GETPID()
      CALL ERR_PASS ( IUER, IER )
      CALL WR_TEXT ( 1, STR, TEMP_NERS_LOCK, IER )
      IF ( IER .NE. 0 ) THEN
           CALL ERR_LOG ( 4342, IUER, 'NERS_FETCH', 'Error in an '// &
     &         'attempt to write lock file '//TEMP_NERS_LOCK )
           RETURN 
      END IF      
      WRITE ( UNIT=CONN_TIMEOUT_STR, FMT='(F12.1)' ) NERS%CNF%CONN_TIMEOUT 
      WRITE ( UNIT=READ_TIMEOUT_STR, FMT='(F12.1)' ) NERS%CNF%READ_TIMEOUT 
      CONN_TIMEOUT_STR = ADJUSTL ( CONN_TIMEOUT_STR )
      READ_TIMEOUT_STR = ADJUSTL ( READ_TIMEOUT_STR )
!
      DO 420 J2=1,NERS%CNF%N_TRIES
         DO 430 J3=1,NERS%CNF%N_URL
            COM_STR = 'wget'// &
     &                ' -c'// &
     &                ' --no-use-server-timestamps'// &
     &                ' --dns-timeout='//TRIM(CONN_TIMEOUT_STR)// &
     &                ' --connect-timeout='//TRIM(CONN_TIMEOUT_STR)// &
     &                ' --read-timeout='//TRIM(READ_TIMEOUT_STR)// &
     &                ' -O '//TRIM(TEMP_NERS_FCS)// &
     &                ' '//TRIM(NERS%CNF%URL(J2))//' > '// &
     &                ' '//TRIM(TEMP_NERS_LOG)//' 2>&1'
            IS = SYSTEM ( TRIM(COM_STR)//CHAR(0) )
            IF ( IS == 0 ) THEN
!
! -------------- Fine! We have finished downloading
!
                 CALL RENAME ( TRIM(TEMP_NERS_FCS)//CHAR(0), TRIM(NERS%CNF%FCS_FILE)//CHAR(0) )
!
! -------------- Remove log and lock files
!
                 CALL UNLINK ( TRIM(TEMP_NERS_LOG)//CHAR(0) )
                 CALL UNLINK ( TRIM(TEMP_NERS_LOCK)//CHAR(0) )
                 CALL ERR_LOG ( 0, IUER )
                 RETURN 
            END IF
 430     CONTINUE 
         IF ( J2 .NE. NERS%CNF%N_TRIES ) THEN
              CALL SLEEP ( IDNINT ( NERS%CNF%CONN_TIMEOUT ) )
         END IF
 420  CONTINUE 
!
      CALL ERR_LOG ( 4343, IUER, 'NERS_FETCH', 'Error in an attempt to '// &
     &    'download EOP from the remote server' )
      IS = SYSTEM ( 'cat '//TRIM(TEMP_NERS_LOG)//CHAR(0) )
!
! -------------- Remove log and lock files
!
      CALL UNLINK ( TRIM(TEMP_NERS_LOG)//CHAR(0) )
      CALL UNLINK ( TRIM(TEMP_NERS_LOCK)//CHAR(0) )
!
      RETURN
      END  SUBROUTINE  NERS_FETCH  !#!#
